@@ -29,6 +29,8 @@
 The bootloader dialog is used to update the Crazyflie firmware and to
 read/write the configuration block in the Crazyflie flash.
 """
+from __future__ import annotations
+
 from cflib.bootloader import Bootloader
 from cfclient.ui.connectivity_manager import ConnectivityManager
 
@@ -36,6 +38,7 @@ import tempfile
 import logging
 import json
 import os
+import re
 import threading
 from urllib.request import urlopen
 from urllib.error import URLError
@@ -82,7 +85,6 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
         self.setupUi(self)
 
         self.tabName = "Service"
-        self.menuName = "Service"
 
         # self.tabWidget = tabWidget
         self._helper = helper
@@ -119,6 +121,7 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
         self._state = self.UIState.DISCONNECTED
 
         self._releases = {}
+        self._platform_widget_names = {}
         self._release_firmwares_found.connect(self._populate_firmware_dropdown)
         self._release_downloaded.connect(self.release_zip_downloaded)
         self.firmware_downloader = FirmwareDownloader(self._release_firmwares_found, self._release_downloaded)
@@ -127,6 +130,8 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
         self.firmware_downloader.start()
         self.clt.start()
 
+        self._platform_filter_checkboxes = []
+
     def _ui_connection_fail(self, message):
         self._cold_boot_error_message = message
         self.setUiState(self.UIState.DISCONNECTED)
@@ -239,16 +244,71 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
         """ Callback from firmware-downloader that retrieves all
             the latest firmware-releases.
         """
+        platforms = set()
         for release in releases:
             release_name = release[0]
-            for download in release[1:]:
+            downloads = release[1:]
+
+            downloads.sort(key=self.download_sorter)
+
+            for download in downloads:
                 download_name, download_link = download
-                widget_name = '%s - %s' % (release_name, download_name)
-                self._releases[widget_name] = download_link
-                self.firmwareDropdown.addItem(widget_name)
+                platform = self._extract_platform(download_name)
+                # Ignore old releases that do not use the standard file naming convention
+                if platform:
+                    widget_name = '%s - %s' % (release_name, download_name)
+                    if platform not in self._platform_widget_names:
+                        self._platform_widget_names[platform] = []
+                    self._platform_widget_names[platform].append(widget_name)
+                    self._releases[widget_name] = download_link
+
+                    platforms.add(platform)
+
+        for platform in sorted(platforms, reverse=True):
+            radio_button = QtWidgets.QRadioButton(platform)
+
+            # Use cf2 as default platform
+            if platform == 'cf2':
+                radio_button.setChecked(True)
+
+            radio_button.toggled.connect(self._update_firmware_dropdown)
+
+            self._platform_filter_checkboxes.append(radio_button)
+            self.filterLayout.insertWidget(0, radio_button)
+
+        self._update_firmware_dropdown(True)
+
+    def _update_firmware_dropdown(self, active: bool):
+        if active:
+            platform = None
+            for button in self._platform_filter_checkboxes:
+                if button.isChecked():
+                    platform = button.text()
+
+            if platform:
+                self.firmwareDropdown.clear()
+                for widget_name in self._platform_widget_names[platform]:
+                    self.firmwareDropdown.addItem(widget_name)
+
+    def _extract_platform(self, download_name: str) -> str | None:
+        # Download name is something like 'firmware-cf2-2022.12.zip'
+        found = re.search('firmware-(\\w+)-', download_name)
+        if found:
+            groups = found.groups()
+            if len(groups) == 1:
+                return groups[0]
+        return None
+
+    def download_sorter(self, element):
+        '''Sort downloads to display cf2 before bolt and tag'''
+        name = element[0]
+        if 'cf2' in name:
+            return '0' + name
+        else:
+            return '1' + name
 
     def release_zip_downloaded(self, release_name, release_path):
-        """ Callback when a release is succesfully downloaded and
+        """ Callback when a release is successfully downloaded and
             save to release_path.
         """
         self.downloadStatus.setText('Downloaded')
@@ -331,7 +391,7 @@ class BootloaderDialog(QtWidgets.QWidget, service_dialog_class):
         logger.debug("Status: [%s] | %d", status, progress)
         self.statusLabel.setText('Status: <b>' + status + '</b>')
         if progress >= 0:
-            self.progressBar.setValue(progress)
+            self.progressBar.setValue(int(progress))
 
     def initiateColdboot(self):
         self.clt.initiateColdBootSignal.emit("radio://0/100")
@@ -441,8 +501,7 @@ class FirmwareDownloader(QThread):
         self._qtsignal_get_release = qtsignal_get_release
 
         self._tempDirectory = tempfile.TemporaryDirectory()
-        self._filepath = os.path.join(self._tempDirectory.name,
-                                      'tmp.zip')
+
         self.moveToThread(self)
 
     def get_firmware_releases(self):
@@ -476,8 +535,7 @@ class FirmwareDownloader(QThread):
             if release_name:
                 releases = [release_name]
                 for download in release['assets']:
-                    releases.append(
-                        (download['name'], download['browser_download_url']))
+                    releases.append((download['name'], download['browser_download_url']))
                 release_list.append(releases)
 
         if release_list:
@@ -489,27 +547,28 @@ class FirmwareDownloader(QThread):
         """ Downloads the given release and calls the callback signal
             if successful.
         """
+        filepath = os.path.join(self._tempDirectory.name, release_name.split(' ')[-1])
         try:
             # Check if we have an old file saved and if so, ensure it's a valid
             # zipfile and then call signal
-            with open(self._filepath, 'rb') as f:
+            with open(filepath, 'rb') as f:
                 previous_release = zipfile.ZipFile(f)
                 # testzip returns None if it's OK.
                 if previous_release.testzip() is None:
                     logger.info('Using same firmware-release file at'
-                                '%s' % self._filepath)
-                    signal.emit(release_name, self._filepath)
+                                '%s' % filepath)
+                    signal.emit(release_name, filepath)
                     return
         except FileNotFoundError:
             try:
                 # Fetch the file with a new web request and save it to
                 # a temporary file.
                 with urlopen(url) as response:
-                    with open(self._filepath, 'wb') as release_file:
+                    with open(filepath, 'wb') as release_file:
                         release_file.write(response.read())
                     logger.info('Created temporary firmware-release file at'
-                                '%s' % self._filepath)
-                    signal.emit(release_name, self._filepath)
+                                '%s' % filepath)
+                    signal.emit(release_name, filepath)
             except URLError:
                 logger.warning('Failed to make web request to get requested'
                                ' firmware-release')
